import { SvelteComponentTyped } from "svelte";
import type { DataTypes, RuleDefinition } from "./common";
declare const __propDef: {
    props: {
        /** Two-way binding for controlling and reading the Dropdown selection
        - see {@link components/use-dropdown} */ selected?: string | string[] | undefined;
        /** Two-way binding for controlling and reading the Modal state
        - see {@link components/use-modal} */ active?: boolean | undefined;
        /** Two-way binding for controlling and reading the Calendar date, time, or datetime
        - see {@link components/use-calendar} */ date?: Date | undefined;
        /** Two-way binding for controlling and reading the Slider value
        - see {@link components/use-slider} */ position?: number | undefined;
        /** Two-way binding for controlling and reading input / textarea value.
        May be used together with {@link components/use-format}. */ value?: DataTypes | undefined;
        /** Optional value validator. Uses Semantic UI validator syntax.
        The same as {@link components/use-validate}.
        Ususally takes the value generated with {@link data/helpers.rule}. */ validate?: RuleDefinition | undefined;
        /** Id of the input element, takes precendence over tag position */ forId?: string | undefined;
    };
    events: {
        [evt: string]: CustomEvent<any>;
    };
    slots: {};
};
export type DataBindProps = typeof __propDef.props;
export type DataBindEvents = typeof __propDef.events;
export type DataBindSlots = typeof __propDef.slots;
/**
 * Svelte data binder for Semantic-UI `Calendar`, `Dropdown`, `Modal`, `Slider` components;
 * as well as `input` and `textarea` elements.
 * The <Data> tag must be a child of the corresponding Semantic UI component
 * or immeditely follow the `input` or `textarea`.
 *
 * ### calendar
 * Takes `date` binding of Date type, which may represent calendar date,
 * time, or date-and-time.
 * ```
 * <div class="ui calendar" use:calendar>
 *     <Data bind:date={alarm} />
 *     <div class="ui input right icon">
 *         <i class="clock outline icon" />
 *         <input type="text" placeholder="Time" />
 *     </div>
 * </div>
 * ```
 *
 * ### dropdown
 * Type of `selected` binding is `string` for a select, or `string[]` for a multi-select.
 * If an invalid value is provided, select is reset, multi-select values are removed.
 * ```
 * <select class="ui selection dropdown" use:dropdown>
 * <Data bind:selected={gender} />
 * <option value="M">Male</option>
 * <option value="F">Female</option>
 * </select>
 * ```
 *
 * ### modal
 * Changing the boolean variable bound to `active` will show or hide the dialogue.
 * ```
 * <div id="md" class="ui modal page" use:modal>
 * <Data bind:active={show} />
 * ...
 * </div>
 * ```
 *
 * ### slider
 * Takes `position` binding of type number.
 * ```
 * <div class="ui labeled ticked slider" use:slider>
 * <Data bind:position={num} />
 * </div>
 * ```
 *
 * ### input / textarea
 * Input and textarea have two different ways to bind data`:` <br />
 * `value` binding in the `<input>` / textarea gets raw text value and is updated immediately. <br />
 * 🔥 `parsed` binding in `<Data>` gets parsed typed data and is updated on blur.
 * ```
 * <input bind:value={text} />
 * <input /><Data bind:parsed={typed} />
 * ```
 *
 * ### checkbox / radio
 * Svelte allows to bind checkbox ands radio inputs in two different ways:
 * ```
 * <input type="checkbox" bind:checked={boolVal} />
 * <input type="radio" bind:checked={boolValTwo} />
 *
 * <input type="checkbox" bind:group={arrayVal} />
 * <input type="radio" bind:group={strVal} />
 * ```
 *
 * ### validation
 * `validate` param in `<Data>` validates the formatted text (not typed data).<br />
 * For `<input>` / textarea / select `use:validate` on the element will produce the same result.
 * ```
 * <input bind:value={text} use:validate={...} />
 * <input /><Data bind:parsed={val} validate={...}>
 * ```
 */
export default class DataBind extends SvelteComponentTyped<DataBindProps, DataBindEvents, DataBindSlots> {
}
export {};
